#include "ColectionItemDisplay.hpp"
#include <QPainter>
#include "ColectionItem.hpp"
#include <QLinearGradient>
#include <QPropertyAnimation>
#include <QSequentialAnimationGroup>
#include <QGraphicsSceneWheelEvent>
#include "ScrollButton.hpp"
#include "SlideIndycator.hpp"

QRectF ColectionItemDisplay::boundingRect() const
{
    return QRectF(0.0f,0.0f,myWidth,myHeight);
}


void ColectionItemDisplay::populate(QList<QPair<int,QString> > list)
{
    clear();
    for(int i = 0; i < list.count(); ++i)
    {
        ColectionItem* item = myPrototype->newInstance(myWidth - up->boundingRect().width() -2.0f,
                                                       list.at(i).first,list.at(i).second,this);
        items.append(item);
        connect(item,SIGNAL(clicked(int)),this,SIGNAL(itemClicked(int)));
    }
    filterItems();
}

void ColectionItemDisplay::clear()
{
    qDeleteAll(items);
    items.clear();
    filteredItems.clear();
    setSlideIndycator();
}

void ColectionItemDisplay::setSize(qreal width,qreal height)
{
    myWidth = width;
    myHeight = height;
    for(int i = 0; i < items.count(); ++i)
        items.at(i)->setWidth(myWidth - up->boundingRect().width() - 2.0f);
    if(filteredItems.count() > 0)
    {
        while(filteredItems.last()->pos().y() < myHeight - filteredItems.last()->boundingRect().height() &&
              filteredItems.first()->pos().y() < 0.0f)
        {
            for(int i = 0; i < filteredItems.count(); ++i)
                filteredItems.at(i)->setVPos(items.at(i)->getVPos() + filteredItems.at(i)->boundingRect().height());
        }
    }
    up->setPos(width - up->boundingRect().width(),0.0f);
    down->setPos(width - up->boundingRect().width(),height - up->boundingRect().height());
    indycator->setPos(myWidth - indycator->boundingRect().width(),up->boundingRect().height());
    setSlideIndycator();
}

void ColectionItemDisplay::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    painter->setPen(Qt::transparent);
    painter->setBrush(QBrush(QColor(10,10,10,255)));
    painter->drawRect(0.0f,0.0f,myWidth,myHeight/* - 48.0f*/);
}

void ColectionItemDisplay::wheelEvent(QGraphicsSceneWheelEvent* event)
{
    event->accept();
    int delta = event->delta();
    if(delta < 0)//scrollUP
    {
        delta *= -1;
        delta /= 120;
        for(int i = 0; i < delta; ++i)
            scrollUp();
    }
    else
    {
        delta /= 120;
        for(int i = 0; i < delta; ++i)
            scrollDown();
    }
}

void ColectionItemDisplay::scrollUp()
{
    if(filteredItems.count() > 0)
    {
        if(filteredItems.last()->pos().y() > boundingRect().height() - filteredItems.last()->boundingRect().height())
        {
            for(int i = 0; i < filteredItems.count(); ++i)
                filteredItems.at(i)->slideUp();
            indycator->tickDown();
        }
    }
}

void ColectionItemDisplay::scrollDown()
{
    if(filteredItems.count() > 0)
    {
        if(filteredItems.first()->pos().y() < 0.0f)
        {
            for(int i = 0; i < filteredItems.count(); ++i)
                filteredItems.at(i)->slideDown();
            indycator->tickUp();
        }
    }
}

void ColectionItemDisplay::scroll(int ticks)
{
    if(ticks < 0)//scrollUP
    {
        ticks *= -1;
        for(int i = 0; i < ticks; ++i)
            scrollUp();
    }
    else
    {
        for(int i = 0; i < ticks; ++i)
            scrollDown();
    }
}

void ColectionItemDisplay::setFilter(const QString& filter)
{
    currentFilter = filter;
    filterItems();
}

void ColectionItemDisplay::aniStoped()
{
    if(group)
    {
        group->deleteLater();
        group = NULL;
    }
}

void ColectionItemDisplay::setSlideIndycator()
{
    int ticks = 1;
    int currentTick = 0;
    for(int i = 0; i < filteredItems.count(); ++i)
    {
        if( filteredItems.at(i)->pos().y() < 0)
        {
            ++ticks;
            ++currentTick;
        }
        else if(filteredItems.at(i)->pos().y() > myHeight - filteredItems.last()->boundingRect().height())
            ++ticks;
    }
    indycator->setParametres(myHeight - (up->boundingRect().height() * 2),ticks,currentTick);
}

void ColectionItemDisplay::filterItems()
{
    if(currentFilter.isEmpty())
    {
        filteredItems = items;
        for(int i = 0; i < items.count(); ++i)
            items.at(i)->show();
    }
    else
    {
        filteredItems.clear();
        for(int i = 0; i < items.count(); ++i)
        {
            if(items.at(i)->getText().contains(currentFilter,Qt::CaseInsensitive))
            {
                filteredItems.append(items.at(i));
                items.at(i)->show();
            }
            else
                items.at(i)->hide();
        }
    }
    if(group)
        group->stop();
    group = new QSequentialAnimationGroup(this);
    for(int i = 0; i < filteredItems.count(); ++i)
    {
        qreal hPos = i * (filteredItems.at(i)->boundingRect().height() + 4.0f);
        if(hPos < boundingRect().height())
        {
            filteredItems.at(i)->setPos(-1 * filteredItems.at(i)->boundingRect().width(),hPos);
            QPropertyAnimation* ani = new QPropertyAnimation(filteredItems.at(i),"hPos");
            ani->setEndValue(0.0f);
            ani->setEasingCurve(QEasingCurve::InOutQuad);
            ani->setDuration(25);
            group->addAnimation(ani);
        }
        else
        {
            filteredItems.at(i)->setPos(0.0f,hPos);
        }
    }
    group->start();
    connect(group,SIGNAL(finished()),this,SLOT(aniStoped()));
    setSlideIndycator();
}

ColectionItemDisplay::ColectionItemDisplay(ColectionItem* prototype,qreal width,qreal height,
                                           QGraphicsItem* parent)
    :QGraphicsObject(parent),myPrototype(prototype),myWidth(width),myHeight(height)
{
    setFlag(QGraphicsItem::ItemClipsChildrenToShape,true);

    up = new ScrollButton(Scroll::up,this);
    up->setPos(width - up->boundingRect().width(),0.0f);
    connect(up,SIGNAL(scroll()),this,SLOT(scrollDown()));

    down = new ScrollButton(Scroll::down,this);
    down->setPos(width - up->boundingRect().width(),height - up->boundingRect().height());
    connect(down,SIGNAL(scroll()),this,SLOT(scrollUp()));

    indycator = new SlideIndycator(myHeight - (down->boundingRect().height() * 2),this);
    indycator->setPos(myWidth - indycator->boundingRect().width(),up->boundingRect().height());
    connect(indycator,SIGNAL(scroll(int)),this,SLOT(scroll(int)));
    currentFilter = "";
    group = NULL;
}

ColectionItemDisplay::~ColectionItemDisplay()
{
    delete myPrototype;
    if(group)
        group->stop();
    delete group;
}
