#include "PlayListDisplay.hpp"
#include "PlayListItem.hpp"
#include <QPainter>
#include <QLinearGradient>
#include <QPropertyAnimation>
#include <QGraphicsSceneWheelEvent>
#include "../Display/ScrollButton.hpp"
#include "../Display/SlideIndycator.hpp"
#include <QGraphicsSceneDragDropEvent>
#include <QMimeData>
#include <QPropertyAnimation>
#include <QSequentialAnimationGroup>
#include <QDataStream>

QRectF PlayListDisplay::boundingRect() const
{
    return QRectF(0.0f,0.0f,myWidth,myHeight);
}


void PlayListDisplay::populate(TrackInfoList list, int index)
{
    for(int i = 0; i < items.count(); ++i)
        list.removeAll(items.at(i)->getTrackInfo());
    if(list.count() > 0)
    {
        if(index == -1 || index >= items.count())
            append(list);
        else
            insertAfter(list,index);
    }
}

void PlayListDisplay::clear()
{
    qDeleteAll(items);
    items.clear();
    currentListIndex = 0;
    setSlideIndycator();
}

void PlayListDisplay::setSize(qreal width,qreal height)
{
    myWidth = width;
    myHeight = height;
    for(int i = 0; i < items.count(); ++i)
        items.at(i)->setWidth(myWidth - up->boundingRect().width() - 2.0f);
    if(items.count() > 0)
    {
        while(items.last()->pos().y() < myHeight - items.last()->boundingRect().height() &&
              items.first()->pos().y() < 0.0f)
        {
            for(int i = 0; i < items.count(); ++i)
                items.at(i)->setVPos(items.at(i)->getVPos() + items.at(i)->boundingRect().height());
        }
    }
    up->setPos(width - up->boundingRect().width(),0.0f);
    down->setPos(width - up->boundingRect().width(),height - up->boundingRect().height());
    indycator->setPos(myWidth - indycator->boundingRect().width(),up->boundingRect().height());
    setSlideIndycator();
}

QString PlayListDisplay::getCurrentPath()
{
    QString path;
    markCurrent();
    if(currentListIndex >= 0 && currentListIndex < items.count())
        path = items.at(currentListIndex)->getPath();
    return path;
}

QString PlayListDisplay::next()
{
    if(currentListIndex < items.count() - 1)
        ++currentListIndex;
    else
        currentListIndex = 0;
    markCurrent();
    return getCurrentPath();
}

QString PlayListDisplay::previous()
{
    if(currentListIndex > 0)
        --currentListIndex;
    else if(items.count() > 0)
        currentListIndex = items.count() - 1;
    markCurrent();
    return getCurrentPath();
}

void PlayListDisplay::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    painter->setPen(Qt::transparent);
    painter->setBrush(QBrush(QColor(10,10,10,255)));
    painter->drawRect(0.0f,0.0f,myWidth,myHeight);
    if(drawDropLine)
    {
        QPen pen;
        pen.setColor(Qt::darkGray);
        pen.setWidth(2);
        painter->setPen(pen);
        painter->drawLine(QPointF(0.0f,dropLineY),
                          QPointF(boundingRect().width() - up->boundingRect().width() - 2.0f,
                                  dropLineY));
    }
}

void PlayListDisplay::wheelEvent(QGraphicsSceneWheelEvent* event)
{
    event->accept();
    int delta = event->delta();
    if(delta < 0)//scrollUP
    {
        delta *= -1;
        delta /= 120;
        for(int i = 0; i < delta; ++i)
            scrollUp();
    }
    else
    {
        delta /= 120;
        for(int i = 0; i < delta; ++i)
            scrollDown();
    }
}

void PlayListDisplay::dragEnterEvent(QGraphicsSceneDragDropEvent* event)
{
    if(event->mimeData()->hasFormat("VulcanoPlayer/TrackIndex") ||
       event->mimeData()->hasFormat("VulcanoPlayer/AlbumIndex") ||
       event->mimeData()->hasFormat("VulcanoPlayer/ArtistIndex") ||
       event->mimeData()->hasFormat("VulcanoPlayer/PlayListItem"))
    {
        event->setAccepted(true);
        drawDropLine = true;
        setDropIndycator(event->scenePos());
    }
    else
        event->setAccepted(false);
}

void PlayListDisplay::dragLeaveEvent(QGraphicsSceneDragDropEvent*)
{
    drawDropLine = false;
    update();
}

void PlayListDisplay::dragMoveEvent(QGraphicsSceneDragDropEvent* event)
{
    setDropIndycator(event->scenePos());
}

void PlayListDisplay::dropEvent(QGraphicsSceneDragDropEvent* event)
{
    if(event->mimeData()->hasFormat("VulcanoPlayer/TrackIndex"))
    {
        int index = event->mimeData()->data("VulcanoPlayer/TrackIndex").toInt();
        emit requestTrack(index,dropIndex);
    }
    else if(event->mimeData()->hasFormat("VulcanoPlayer/AlbumIndex"))
    {
        int index = event->mimeData()->data("VulcanoPlayer/AlbumIndex").toInt();
        emit requestTracks(true,index,dropIndex);
    }
    else if(event->mimeData()->hasFormat("VulcanoPlayer/ArtistIndex"))
    {
        int index = event->mimeData()->data("VulcanoPlayer/ArtistIndex").toInt();
        emit requestTracks(false,index,dropIndex);
    }
    else if(event->mimeData()->hasFormat("VulcanoPlayer/PlayListItem"))
    {
        TrackInfo info;
        QDataStream stream(event->mimeData()->data("VulcanoPlayer/PlayListItem"));
        stream >> info;
        PlayListItem* item = NULL;
        for(int i = 0; i < items.count(); ++i)
        {
            if(items.at(i)->getTrackInfo() == info)
                item = items.at(i);
        }
        if(item)
            transferItem(item,dropIndex);
    }
    drawDropLine = false;
    update();
}

void PlayListDisplay::scrollUp()
{
    if(items.count() > 0)
    {
        if(items.last()->pos().y() > boundingRect().height() - items.last()->boundingRect().height())
        {
            for(int i = 0; i < items.count(); ++i)
                items.at(i)->slideUp();
            indycator->tickDown();
        }
    }
}

void PlayListDisplay::scrollDown()
{
    if(items.count() > 0)
    {
        if(items.first()->pos().y() < 0.0f)
        {
            for(int i = 0; i < items.count(); ++i)
                items.at(i)->slideDown();
            indycator->tickUp();
        }
    }
}

void PlayListDisplay::scroll(int ticks)
{
    if(ticks < 0)//scrollUP
    {
        ticks *= -1;
        for(int i = 0; i < ticks; ++i)
            scrollUp();
    }
    else
    {
        for(int i = 0; i < ticks; ++i)
            scrollDown();
    }
}

void PlayListDisplay::itemClickedAction(PlayListItem* item)
{
    if(item)
    {
        emit itemClicked(item->getPath());
        currentListIndex = items.indexOf(item);
        markCurrent();
    }
}

void PlayListDisplay::setSlideIndycator()
{
    int ticks = 1;
    int currentTick = 0;
    for(int i = 0; i < items.count(); ++i)
    {
        if( items.at(i)->pos().y() < 0)
        {
            ++ticks;
            ++currentTick;
        }
        else if(items.at(i)->pos().y() > myHeight - items.last()->boundingRect().height())
            ++ticks;
    }
    indycator->setParametres(myHeight - (up->boundingRect().height() * 2),ticks,currentTick);
}

void PlayListDisplay::append(TrackInfoList list)
{
    qreal ypos = 0.0f;
    if(items.count() > 0)
        ypos = items.last()->getVPos() + items.last()->boundingRect().height()  + 4.0f;
    QSequentialAnimationGroup* group = new QSequentialAnimationGroup(this);
    for(int i = 0; i < list.count(); ++i)
    {
        PlayListItem* item = new PlayListItem(myWidth - up->boundingRect().width() - 2.0f,
                                              list.at(i),this);
        items.append(item);
        connect(item,SIGNAL(clicked(PlayListItem*)),this,SLOT(itemClickedAction(PlayListItem*)));
        connect(item,SIGNAL(remove(PlayListItem*)),this,SLOT(removeItem(PlayListItem*)));
        if(ypos <= boundingRect().height())
        {
            item->setPos(-1 * item->boundingRect().width(),ypos);
            QPropertyAnimation* ani = new QPropertyAnimation(item,"hPos");
            ani->setEndValue(0.0f);
            ani->setEasingCurve(QEasingCurve::InOutQuad);
            ani->setDuration(25);
            group->addAnimation(ani);
        }
        else
        {
            item->setPos(0.0f,ypos);
        }
        ypos += item->boundingRect().height()  + 4.0f;
    }
    setSlideIndycator();
    group->start(QAbstractAnimation::DeleteWhenStopped);
}

void PlayListDisplay::insertAfter(TrackInfoList list,int index)
{
    qreal ypos = 0.0f;
    if(index > -1 && index < items.count())
    {
        ypos = items.at(index)->getVPos() + items.at(index)->boundingRect().height() + 4.0f;
        for(int i = index + 1; i < items.count(); ++i)
            items.at(i)->setVPos(items.at(i)->getVPos() +
                                 (items.at(i - 1)->boundingRect().height() + 4.0f) * list.count());
    }
    if(currentListIndex > index)
    {
        currentListIndex += list.count();
    }
    QSequentialAnimationGroup* group = new QSequentialAnimationGroup(this);
    for(int i = 0; i < list.count(); ++i)
    {
        PlayListItem* item = new PlayListItem(myWidth - up->boundingRect().width() - 2.0f,
                                              list.at(i),this);
        ++index;
        items.insert(index,item);
        connect(item,SIGNAL(clicked(PlayListItem*)),this,SLOT(itemClickedAction(PlayListItem*)));
        connect(item,SIGNAL(remove(PlayListItem*)),this,SLOT(removeItem(PlayListItem*)));
        if(ypos <= boundingRect().height())
        {
            item->setPos(-1 * item->boundingRect().width(),ypos);
            QPropertyAnimation* ani = new QPropertyAnimation(item,"hPos");
            ani->setEndValue(0.0f);
            ani->setEasingCurve(QEasingCurve::InOutQuad);
            ani->setDuration(25);
            group->addAnimation(ani);
        }
        else
        {
            item->setPos(0.0f,ypos);
        }
        ypos += item->boundingRect().height() + 4.0f;
    }
    setSlideIndycator();
    group->start(QAbstractAnimation::DeleteWhenStopped);
}

void PlayListDisplay::setDropIndycator(QPointF pos)
{
    if(items.count() > 0)
    {
        qreal posY = pos.y();
        bool found = false;
        for(int i = 0; i < items.count(); ++i)
        {
            if(posY >= items.at(i)->getVPos() && posY <= (items.at(i)->getVPos() +
                                                          items.at(i)->boundingRect().height() + 4.0f))
            {
                found = true;
                dropLineY = items.at(i)->getVPos() + items.at(i)->boundingRect().height() + 2.0f;
                dropIndex = i;
                break;
            }
        }
        if(!found)
        {
            dropLineY = items.last()->getVPos() + items.last()->boundingRect().height() + 2.0f;
            dropIndex = items.count() - 1;
        }
    }
    else
    {
        dropLineY = 1.0f;
        dropIndex = 0;
    }
    update();
}

void PlayListDisplay::markCurrent()
{
    for(int i = 0; i < items.count();++i)
    {
        if(i == currentListIndex)
            items.at(i)->setCurrent(true);
        else
            items.at(i)->setCurrent(false);
    }
}

void PlayListDisplay::transferItem(PlayListItem* item,int after)
{
    if(items.contains(item) && after >= 0 && after < items.count())
    {
        int itemIndex = items.indexOf(item);
        int oldAfter = after;
        if(itemIndex < after) //move down we need to adjust because the after item will move 1 slot up
            --after;
        else if(itemIndex == after) // :| wft IDIOT
            return;
        items.removeOne(item);
        for(int i = itemIndex; i < items.count();   ++i)
            items.at(i)->setVPos(items.at(i)->getVPos() - (item->boundingRect().height() + 4.0f));
        qreal yPos = items.at(after)->getVPos() + (items.at(after)->boundingRect().height() + 4.0f);
        ++after;
        items.insert(after,item);
        item->setVPos(yPos);
        for(int i = after + 1; i < items.count(); ++i)
            items.at(i)->setVPos(items.at(i - 1)->getVPos() + (items.at(i - 1)->boundingRect().height() + 4.0f));
        if(currentListIndex == itemIndex)
            currentListIndex = items.indexOf(item);
        else if(currentListIndex > itemIndex && currentListIndex <= oldAfter)
            --currentListIndex;
        else if(currentListIndex < itemIndex && currentListIndex > oldAfter)
            ++currentListIndex;
    }
}

void PlayListDisplay::removeItem(PlayListItem* item)
{
    if(items.contains(item))
    {
        int index = items.indexOf(item);
        if(index < currentListIndex)
            --currentListIndex;
        else if(index == currentListIndex)
            currentListIndex = -1;
        items.removeOne(item);
        for(int i = index; i < items.count();   ++i)
            items.at(i)->setVPos(items.at(i)->getVPos() - (item->boundingRect().height() + 4.0f));
        delete item;
    }
}

PlayListDisplay::PlayListDisplay(qreal width,qreal height,QGraphicsItem* parent)
    :QGraphicsObject(parent),myWidth(width),myHeight(height)
{
    setFlag(QGraphicsItem::ItemClipsChildrenToShape,true);

    up = new ScrollButton(Scroll::up,this);
    up->setPos(width - up->boundingRect().width(),0.0f);
    connect(up,SIGNAL(scroll()),this,SLOT(scrollDown()));

    down = new ScrollButton(Scroll::down,this);
    down->setPos(width - up->boundingRect().width(),height - up->boundingRect().height());
    connect(down,SIGNAL(scroll()),this,SLOT(scrollUp()));

    indycator = new SlideIndycator(myHeight - (down->boundingRect().height() * 2),this);
    indycator->setPos(myWidth - indycator->boundingRect().width(),up->boundingRect().height());
    connect(indycator,SIGNAL(scroll(int)),this,SLOT(scroll(int)));
    currentFilter = "";
    this->setAcceptDrops(true);
    drawDropLine = false;
    dropLineY = 0.0f;
    dropIndex = 0;
    currentListIndex = 0;
}

PlayListDisplay::~PlayListDisplay()
{
}
