#include "VulcanoWindow.hpp"
#include "Colection/ColectionScanner.hpp"
#include "Display/ColectionToolBar.hpp"
#include "Display/GraphicsToolButton.hpp"
#include "Colection/ScanDialog.hpp"
#include "Colection/Colection.hpp"
#include "Display/ColectionItemDisplay.hpp"
#include "Display/ColectionItem.hpp"
#include "Display/TrackItem.hpp"
#include "Display/FilterInput.hpp"
#include "Display/ArtistAlbumItem.hpp"
#include "PlayList/PlayListDisplay.hpp"
#include "Display/VolumeControl.hpp"
#include "PlayList/SeekerItem.hpp"
#include <QGraphicsScene>
#include <QSettings>
#include <QResizeEvent>

void VulcanoWindow::resizeEvent(QResizeEvent* event)
{
    makeScenes();
    QSettings setting;
    QSize size = event->size();
    setting.setValue("VulcanoPlayer/Size",size);
}

void VulcanoWindow::showEvent(QShowEvent*)
{
    makeScenes();
}

void VulcanoWindow::closeEvent(QCloseEvent*)
{
    mediaObject->stop();
}

void VulcanoWindow::scan()
{
    ScanDialog* dialog = new ScanDialog(this);
    if(dialog->exec())
    {
        TrackInfoList list = dialog->getResault();
        emit add(list);
        artistAlbumDisplay->clear();
        trackDisplay->clear();
    }
    delete dialog;
}

void VulcanoWindow::colectionReady()
{
    if(album)
        emit getAblums();
    else
        emit getArtist();
}

void VulcanoWindow::artistAlbumItemClocked(int index)
{
    if(album)
        emit getTracksFromAlbum(index);
    else
        emit getTracksFromArtist(index);
}

void VulcanoWindow::trackItemClicked(int index)
{
    emit getTrackInfo(index);
    dropIndex = -1;
}

void VulcanoWindow::switchSorting()
{
    artistAlbumDisplay->clear();
    trackDisplay->clear();
    album = !album;
    artistAlbumColectionItem->setIsAlbum(album);
    QSettings settings;
    settings.setValue("colection/AlbumSor",album);
    colectionReady();
}

void VulcanoWindow::playFile(QString path)
{
    mediaObject->stop();
    mediaObject->clearQueue();
    Phonon::MediaSource source(path);
    mediaObject->setCurrentSource(source);
    mediaObject->play();
}

void VulcanoWindow::play()
{
    if(mediaObject->state() == Phonon::PausedState)
        mediaObject->play();
    else if(mediaObject->state() != Phonon::PlayingState)
    {
        QString path = playList->getCurrentPath();
        if(!path.isEmpty())
        {
            Phonon::MediaSource source(path);
            mediaObject->setCurrentSource(source);
            mediaObject->play();
        }
    }
}

void VulcanoWindow::stop()
{
    if(mediaObject->state() == Phonon::PausedState || mediaObject->state() == Phonon::PlayingState)
    {
        mediaObject->stop();
        mediaObject->clearQueue();
        seeker->trackLeanghtChanged(1);
        seeker->currentTimeChanged(0);
    }
}

void VulcanoWindow::pause()
{
    if(mediaObject->state() == Phonon::PlayingState)
        mediaObject->pause();
    else if(mediaObject->state() == Phonon::PausedState)
        mediaObject->play();
}

void VulcanoWindow::next()
{
    if(mediaObject->state() == Phonon::PausedState || mediaObject->state() == Phonon::PlayingState)
    {
        mediaObject->stop();
        mediaObject->clearQueue();
        QString path = playList->next();
        if(!path.isEmpty())
        {
            Phonon::MediaSource source(path);
            mediaObject->setCurrentSource(source);
            mediaObject->play();
        }
    }
}

void VulcanoWindow::previous()
{
    if(mediaObject->state() == Phonon::PausedState || mediaObject->state() == Phonon::PlayingState)
    {
        mediaObject->stop();
        mediaObject->clearQueue();
        QString path = playList->previous();
        if(!path.isEmpty())
        {
            Phonon::MediaSource source(path);
            mediaObject->setCurrentSource(source);
            mediaObject->play();
        }
    }
}

void VulcanoWindow::aboutToFinish()
{
    QString path = playList->next();
    if(!path.isEmpty())
    {
        Phonon::MediaSource source(path);
        mediaObject->enqueue(source);
        seeker->trackLeanghtChanged(1);
        seeker->currentTimeChanged(0);
        if(mediaObject->state() != Phonon::PlayingState)
            mediaObject->play();
    }
}

void VulcanoWindow::mediaStateChanged(Phonon::State state,Phonon::State oldState)
{
    if(state == Phonon::ErrorState && oldState == Phonon::LoadingState)
        next();
}

void VulcanoWindow::gotAlbumsArtis(QList<QPair<int,QString> > list)
{
    artistAlbumDisplay->populate(list);
}

void VulcanoWindow::gotTracks(QList<QPair<int,QString> > list)
{
    trackDisplay->populate(list);
}

void VulcanoWindow::gotTrackInfo(TrackInfo info)
{
    TrackInfoList list;
    list << info;
    playList->populate(list,dropIndex);
}

void VulcanoWindow::gotTracksInfo(TrackInfoList list)
{
    playList->populate(list,dropIndex);
}

void VulcanoWindow::gotTrackRequest(int index,int dropIndex_)
{
    emit getTrackInfo(index);
    dropIndex = dropIndex_;
}

void VulcanoWindow::gotTracksRequest(bool album,int index,int dropIndex_)
{
    if(album)
        emit getTracksInfoFromAlbum(index);
    else
        emit getTracksInfoFromArtist(index);
    dropIndex = dropIndex_;
}


void VulcanoWindow::createToolButtons()
{
    searchButton = colectionBar->addToolButtonLeft(QPixmap(":/eyecandy/scan.png"));
    connect(searchButton,SIGNAL(clicked()),this,SLOT(scan()));

    GraphicsToolButton* colectionSort = colectionBar->addToolButtonLeft(QPixmap(":/eyecandy/sort.png"));
    connect(colectionSort,SIGNAL(clicked()),this,SLOT(switchSorting()));

    GraphicsToolButton* stopButton = colectionBar->addToolButtonRight(QPixmap(":/eyecandy/stop.png"));
    connect(stopButton,SIGNAL(clicked()),this,SLOT(stop()));
    GraphicsToolButton* pauseButton = colectionBar->addToolButtonRight(QPixmap(":/eyecandy/pause.png"));
    connect(pauseButton,SIGNAL(clicked()),this,SLOT(pause()));
    GraphicsToolButton* playButton = colectionBar->addToolButtonRight(QPixmap(":/eyecandy/play.png"));
    connect(playButton,SIGNAL(clicked()),this,SLOT(play()));

    GraphicsToolButton* nextButton = colectionBar->addToolButtonRight(QPixmap(":/eyecandy/next.png"));
    connect(nextButton,SIGNAL(clicked()),this,SLOT(next()));
    GraphicsToolButton* prevButton = colectionBar->addToolButtonRight(QPixmap(":/eyecandy/previous.png"));
    connect(prevButton,SIGNAL(clicked()),this,SLOT(previous()));
}

void VulcanoWindow::makeScenes()
{
    QGraphicsScene* oldScene = ui.toolBarView->scene();
    QGraphicsScene* toolScene = new QGraphicsScene(0.0f,0.0f,ui.toolBarView->width(),
                                                   ui.toolBarView->height());
    if(oldScene)
        oldScene->removeItem(colectionBar);
    colectionBar->setWidth(ui.toolBarView->width());
    toolScene->addItem(colectionBar);
    ui.toolBarView->setScene(toolScene);
    delete oldScene;

    oldScene = ui.colleciotnView->scene();
    qreal displayWidth = static_cast<qreal>(ui.colleciotnView->width())/2.0f;
    qreal displayHeight = static_cast<qreal>(ui.colleciotnView->height() - 20);
    QGraphicsScene* colectionScene = new QGraphicsScene(0.0f,0.0f,ui.colleciotnView->width(),
                                                        ui.colleciotnView->height());
    if(oldScene)
    {
        oldScene->removeItem(artistAlbumDisplay);
        oldScene->removeItem(artisAlbumFilter);
        oldScene->removeItem(trackDisplay);
        oldScene->removeItem(trackFilter);
    }
    qreal resedueWith = ui.colleciotnView->width()%2 == 0 ? 0.0f : 1.0f;
    artistAlbumDisplay->setSize(displayWidth + resedueWith,
                                displayHeight);
    artisAlbumFilter->resize(displayWidth + resedueWith,20.0f);
    artisAlbumFilter->setPos(0.0f,0.0f);
    trackDisplay->setSize(displayWidth,displayHeight);
    trackDisplay->setPos(displayWidth,20.0f);
    trackFilter->resize(displayWidth,20.0f);
    trackFilter->setPos(displayWidth,0.0f);
    ui.colleciotnView->setScene(colectionScene);
    colectionScene->addItem(artisAlbumFilter);
    colectionScene->addItem(trackFilter);
    colectionScene->addItem(artistAlbumDisplay);
    colectionScene->addItem(trackDisplay);
    delete oldScene;

    oldScene = ui.playListView->scene();

    QGraphicsScene* playListScene = new QGraphicsScene(0.0f,0.0f,ui.playListView->width(),
                                                       ui.playListView->height());
    if(oldScene)
    {
        oldScene->removeItem(playList);
        oldScene->removeItem(seeker);
    }
    playList->setSize(static_cast<qreal>(ui.playListView->width()),
                      static_cast<qreal>(ui.playListView->height()) - seeker->boundingRect().height());
    ui.playListView->setScene(playListScene);
    playListScene->addItem(playList);
    seeker->changeWidth(static_cast<qreal>(ui.playListView->width()));
    //seeker->setPos(0.0f,static_cast<qreal>(ui.playListView->height()) - seeker->boundingRect().height());
    playListScene->addItem(seeker);
    delete oldScene;
}

VulcanoWindow::VulcanoWindow(QWidget *parent)
    :QMainWindow(parent)
{
    ui.setupUi(this);

    qRegisterMetaType<TrackInfoList>("TrackInfoList");
    qRegisterMetaType<TrackInfo>("TrackInfo");
    qRegisterMetaTypeStreamOperators<TrackInfo>("TrackInfo");
    qRegisterMetaType<QList<QPair<int,QString> > >("QList<QPair<int,QString> >");

    mediaObject = new Phonon::MediaObject(this);
    mediaObject->setTickInterval(1000);
    connect(mediaObject,SIGNAL(aboutToFinish()),this,SLOT(aboutToFinish()));
    connect(mediaObject,SIGNAL(stateChanged(Phonon::State,Phonon::State)),
            this,SLOT(mediaStateChanged(Phonon::State,Phonon::State)));
    audioOutPut = new Phonon::AudioOutput(Phonon::MusicCategory,this);
    Phonon::createPath(mediaObject,audioOutPut);

    QGraphicsScene* toolScene = new QGraphicsScene(0.0f,0.0f,ui.toolBarView->width(),
                                                   ui.toolBarView->height());
    ui.toolBarView->setScene(toolScene);

    colectionBar = new ColectionToolBar(ui.toolBarView->width());
    colectionBar->setPos(0.0f,0.0f);
    toolScene->addItem(colectionBar);
    createToolButtons();

    VolumeControl* controler = colectionBar->getVolumeControl();
    connect(controler,SIGNAL(slideValueChanged(qreal)),audioOutPut,SLOT(setVolume(qreal)));
    connect(audioOutPut,SIGNAL(volumeChanged(qreal)),controler,SLOT(volumeChanged(qreal)));
    audioOutPut->setVolume(1.0f);

    colection = new Colection();
    connect(this,SIGNAL(add(TrackInfoList)),colection,SLOT(add(TrackInfoList)),Qt::QueuedConnection);
    connect(this,SIGNAL(getAblums()),colection,SLOT(getAlbums()),Qt::QueuedConnection);
    connect(this,SIGNAL(getArtist()),colection,SLOT(getArtists()),Qt::QueuedConnection);
    connect(this,SIGNAL(getTracksFromAlbum(int)),colection,SLOT(getTracksFromAlbum(int)),
            Qt::QueuedConnection);
    connect(this,SIGNAL(getTracksFromArtist(int)),colection,SLOT(getTracksFromArtist(int)),
            Qt::QueuedConnection);
    connect(this,SIGNAL(getTrackInfo(int)),colection,SLOT(getTrackInfo(int)),
            Qt::QueuedConnection);
    connect(this,SIGNAL(getTracksInfoFromAlbum(int)),colection,SLOT(getTracksInfoFromAlbum(int)),
            Qt::QueuedConnection);
    connect(this,SIGNAL(getTracksInfoFromArtist(int)),colection,SLOT(getTracksInfoFromArtist(int)),
            Qt::QueuedConnection);

    connect(colection,SIGNAL(ready()),this,SLOT(colectionReady()),Qt::QueuedConnection);
    connect(colection,SIGNAL(gotAlbums(QList<QPair<int,QString> >)),
            this,SLOT(gotAlbumsArtis(QList<QPair<int,QString> >)),Qt::QueuedConnection);
    connect(colection,SIGNAL(gotArtists(QList<QPair<int,QString> >)),
            this,SLOT(gotAlbumsArtis(QList<QPair<int,QString> >)),Qt::QueuedConnection);
    connect(colection,SIGNAL(gotTracks(QList<QPair<int,QString> >)),
            this,SLOT(gotTracks(QList<QPair<int,QString> >)),Qt::QueuedConnection);
    connect(colection,SIGNAL(gotTrackInfo(TrackInfo)),this,SLOT(gotTrackInfo(TrackInfo)),
            Qt::QueuedConnection);
    connect(colection,SIGNAL(gotTrackFromArtistAlbum(TrackInfoList)),
            this,SLOT(gotTracksInfo(TrackInfoList)),Qt::QueuedConnection);

    qreal displayWidth = static_cast<qreal>(ui.colleciotnView->width()) * 0.5f;
    qreal displayHeight = static_cast<qreal>(ui.colleciotnView->height() - 20.0f);

    artisAlbumFilter = new FilterInput(displayWidth,20.0f);
    artisAlbumFilter->setPos(0.0f,0.0f);
    trackFilter = new FilterInput(displayWidth,20.0f);
    trackFilter->setPos(displayWidth,0.0f);


    artistAlbumColectionItem = new ArtistAlbumItem(1.0f,0,"",true);
    artistAlbumDisplay = new ColectionItemDisplay(artistAlbumColectionItem,
                                                  displayWidth,displayHeight);
    artistAlbumDisplay->setPos(0.0f,20.0f);
    trackDisplay = new ColectionItemDisplay(new TrackItem(1,0,""),
                                            displayWidth,displayHeight);
    trackDisplay->setPos(displayWidth,0.0f);

    connect(artistAlbumDisplay,SIGNAL(itemClicked(int)),this,SLOT(artistAlbumItemClocked(int)));
    connect(artisAlbumFilter,SIGNAL(applayFilter(QString)),artistAlbumDisplay,SLOT(setFilter(QString)));
    connect(trackDisplay,SIGNAL(itemClicked(int)),this,SLOT(trackItemClicked(int)));
    connect(trackFilter,SIGNAL(applayFilter(QString)),trackDisplay,SLOT(setFilter(QString)));

    QGraphicsScene* colectionScene = new QGraphicsScene(0.0f,0.0f,ui.colleciotnView->width(),
                                                        ui.colleciotnView->height());
    ui.colleciotnView->setScene(colectionScene);
    colectionScene->addItem(artisAlbumFilter);
    colectionScene->addItem(trackFilter);
    colectionScene->addItem(artistAlbumDisplay);
    colectionScene->addItem(trackDisplay);
    seeker = new SeekerItem(static_cast<qreal>(ui.playListView->width()));
    connect(mediaObject,SIGNAL(totalTimeChanged(qint64)),seeker,SLOT(trackLeanghtChanged(qint64)));
    connect(mediaObject,SIGNAL(tick(qint64)),seeker,SLOT(currentTimeChanged(qint64)));
    connect(seeker,SIGNAL(tryToJup(qint64)),mediaObject,SLOT(seek(qint64)));

    playList = new PlayListDisplay(static_cast<qreal>(ui.playListView->width()),
                                   static_cast<qreal>(ui.playListView->height()) - seeker->boundingRect().height());

    connect(playList,SIGNAL(requestTrack(int,int)),this,SLOT(gotTrackRequest(int,int)));
    connect(playList,SIGNAL(requestTracks(bool,int,int)),this,SLOT(gotTracksRequest(bool,int,int)));
    connect(playList,SIGNAL(itemClicked(QString)),this,SLOT(playFile(QString)));
    playList->setPos(0.0f,seeker->boundingRect().height());
    seeker->setPos(0.0f,0.0f);
    QGraphicsScene* playListScene = new QGraphicsScene(0.0f,0.0f,ui.playListView->width(),
                                                       ui.playListView->height());
    ui.playListView->setScene(playListScene);
    playListScene->addItem(playList);
    playListScene->addItem(seeker);
    QSettings setting;
    album = setting.value("colection/AlbumSor",true).toBool();
    artistAlbumColectionItem->setIsAlbum(album);
    dropIndex  = 0;
    QSize size = setting.value("VulcanoPlayer/Size",QSize(600,448)).toSize();
    resize(size);
}

VulcanoWindow::~VulcanoWindow()
{
    mediaObject->stop();
    delete colection;
}

void VulcanoWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui.retranslateUi(this);
        break;
    default:
        break;
    }
}
